#!/bin/bash

#/************************************************************************************************
# * Copyright  (c) 2014, 2015 University of Pennsylvania
# *
# * Permission to use, copy, modify, and distribute this software and
# * its documentation for any purpose, without fee, and without a
# * written agreement is hereby granted, provided that the above copyright
# * notice and this paragraph and the following two paragraphs appear in
# * all copies.
# *
# * IN NO EVENT SHALL THE UNIVERSITY OF PENNSYLVANIA BE LIABLE TO ANY PARTY FOR
# * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# * THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF PENNSYLVANIA
# * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# *
# * THE UNIVERSITY OF PENNSYLVANIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
# * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
# * AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON
# * AN "AS IS" BASIS, AND THE UNIVERSITY OF PENNSYLVANIA HAS NO OBLIGATIONS TO
# * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
# ***********************************************************************************************/
#
#/*** run.sh
#	Author: Edin Kadric
#	IC Group, University of Pennsylvania
#
#	This script shows how to run PowerOpt with VTR
#	Available online: http://ic.ese.upenn.edu/distributions/meme_fpga2015/
#	PowerOpt optimizes memory mapping for power reduction
#	See our FPGA 2015 paper:
#		Edin Kadric, David Lakata, Andre Dehon, "Impact of Memory Architecture on FPGA Energy Consumption"
#***/

# Files to use
vtr_path="$HOME/Downloads/vtr_release/vtr_flow"
circuit_path="verilog/"
circuit="mem_2Kx32.v"
arch_path="gen_arch/mem1/"
arch="k4_N10_L1_mem256x32_MEM0_poweropt2_freq4_tech22nm.xml"
cmos_tech_file="tech/22nm_LP.xml"
delayopt=0 # typically 0; 1 means that PowerOpt tricks aren't used
force_low_depth=0 # see PowerOpt.java for explanation; should probably remain 0

# Variables
BENCHMARK=${circuit/.v/}
ABS_PATH=$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )
OUTDIR=runs/${arch}/${circuit}
mkdir -p $OUTDIR

# Step 1: compile PowerOpt.java
echo "Step 1: compile PowerOpt.java"
javac *.java

# Step 2: generate architecture files
echo "Step 2: generate architecture files:"
python gen_arch/arch_gen.py

# Step 3: Read parameters for PowerOpt from architecture file
echo "Step 3: Read parameters for PowerOpt from architecture file"
#Check if given benchmark is PowerOpt; send appropriate parameters to PowerOpt.java (depth, width, mode)
line=$(head -n 1 $arch_path/$arch)
line2=$(head -n 2 $arch_path/$arch)
if [[ $arch =~ .*poweropt.* ]]
then
	phy_depth=`echo $line| cut -d'_' -f 2`
	phy_width=`echo $line| cut -d'_' -f 4`
	if [[ $MEMTWO =~ .*1.* ]]
	then
		phy_depth2=`echo $line2| cut -d'_' -f 6`
		phy_width2=`echo $line2| cut -d'_' -f 8`
	else
		echo "MEMTWO = $MEMTWO"
		phy_depth2=0
		phy_width2=0
	fi
	mode=1
	echo 'Using following depth and width: '
	echo "phy_depth  = $phy_depth"
	echo "phy_width  = $phy_width"
	echo "phy_depth2 = $phy_depth2"
	echo "phy_width2 = $phy_width2"
else
	phy_width=0
	phy_depth=0
	phy_depth2=0
	phy_width2=0
	mode=0
	echo 'Mode = 0;'
fi

# Step 4: Call PowerOpt
echo "Step 4: Call PowerOpt"
cp $circuit_path/$circuit $OUTDIR/$circuit
java -cp . PowerOpt $OUTDIR/ $BENCHMARK $phy_depth $phy_width $mode $delayopt $force_low_depth $phy_depth2 $phy_width2 || { echo -e "\nError in PowerOpt, exiting"; exit 1; }
mv $OUTDIR/${BENCHMARK}Opt.v $OUTDIR/${BENCHMARK}.v  || { echo -e "\nError in PowerOpt, exiting"; exit 1; }

#Step 5: Run VTR
echo "Step 5: Run VTR"
perl $vtr_path/scripts/run_vtr_flow.pl $OUTDIR/$circuit $arch_path/$arch -keep_intermediate_files -temp_dir $OUTDIR -power -cmos_tech $cmos_tech_file || { echo -e "\nError in VTR"; exit 1; }

#Done
echo "Done."
